@@ -2,4 +2,7 @@
 This package contains an implementation of the Distributed Networking
 API, a high-level networking system that automatically propagates
 changes made on distributed objects to interested clients.
+
+See the :ref:`distributed-networking` section of the Programming Guide to
+learn more about the distributed networking system.
 """
