@@ -1,6 +1,16 @@
 """Contains the BufferViewer class, which is used as a debugging aid
 when debugging render-to-texture effects.  It shows different views at
-the bottom of the screen showing the various render targets."""
+the bottom of the screen showing the various render targets.
+
+When using ShowBase, the normal way to enable the BufferViewer is using the
+following code::
+
+    base.bufferViewer.toggleEnable()
+
+Or, you can enable the following variable in your Config.prc::
+
+    show-buffers true
+"""
 
 __all__ = ['BufferViewer']
 
@@ -11,6 +21,7 @@ from direct.directnotify.DirectNotifyGlobal import *
 from direct.showbase.DirectObject import DirectObject
 import math
 
+
 class BufferViewer(DirectObject):
     notify = directNotify.newCategory('BufferViewer')
 
@@ -100,11 +111,13 @@ class BufferViewer(DirectObject):
 
     def setPosition(self, pos):
         """Set the position of the cards.  The valid values are:
-        * llcorner - put them in the lower-left  corner of the window
-        * lrcorner - put them in the lower-right corner of the window
-        * ulcorner - put them in the upper-left  corner of the window
-        * urcorner - put them in the upper-right corner of the window
-        * window   - put them in a separate window
+
+        - *llcorner* - put them in the lower-left  corner of the window
+        - *lrcorner* - put them in the lower-right corner of the window
+        - *ulcorner* - put them in the upper-left  corner of the window
+        - *urcorner* - put them in the upper-right corner of the window
+        - *window* - put them in a separate window
+
         The initial value is 'lrcorner'."""
         valid=["llcorner","lrcorner","ulcorner","urcorner","window"]
         if (valid.count(pos)==0):
@@ -119,11 +132,13 @@ class BufferViewer(DirectObject):
 
     def setLayout(self, lay):
         """Set the layout of the cards.  The valid values are:
-        * vline - display them in a vertical line
-        * hline - display them in a horizontal line
-        * vgrid - display them in a vertical grid
-        * hgrid - display them in a horizontal grid
-        * cycle - display one card at a time, using selectCard/advanceCard
+
+        - *vline* - display them in a vertical line
+        - *hline* - display them in a horizontal line
+        - *vgrid* - display them in a vertical grid
+        - *hgrid* - display them in a horizontal grid
+        - *cycle* - display one card at a time, using selectCard/advanceCard
+
         The default value is 'hline'."""
         valid=["vline","hline","vgrid","hgrid","cycle"]
         if (valid.count(lay)==0):
@@ -225,10 +240,10 @@ class BufferViewer(DirectObject):
             offsetx = (ringoffset[ring]*2.0) / float(sizex)
             offsety = (ringoffset[ring]*2.0) / float(sizey)
             bright = ringbright[ring]
-            vwriter.addData3f(-1-offsetx, 0, -1-offsety)
-            vwriter.addData3f(1+offsetx, 0, -1-offsety)
-            vwriter.addData3f(1+offsetx, 0,  1+offsety)
-            vwriter.addData3f(-1-offsetx, 0,  1+offsety)
+            vwriter.addData3f(Vec3F.rfu(-1 - offsetx, 0, -1 - offsety))
+            vwriter.addData3f(Vec3F.rfu( 1 + offsetx, 0, -1 - offsety))
+            vwriter.addData3f(Vec3F.rfu( 1 + offsetx, 0,  1 + offsety))
+            vwriter.addData3f(Vec3F.rfu(-1 - offsetx, 0,  1 + offsety))
             cwriter.addData3f(bright, bright, bright)
             cwriter.addData3f(bright, bright, bright)
             cwriter.addData3f(bright, bright, bright)
