@@ -1,5 +1,6 @@
 """
 This package contains various modules that provide a drop-in substitute
 for some of the built-in Python modules.  These substitutes make better
-use of Panda3D's virtual file system and threading system.
+use of Panda3D's :ref:`Virtual File System <virtual-file-system>` and
+:ref:`threading` system.
 """
