@@ -1,4 +1,4 @@
-"""TaskManagerGlobal module: contains the global task manager"""
+"""Contains the global :class:`~.Task.TaskManager` object."""
 
 __all__ = ['taskMgr']
 
