@@ -1442,7 +1442,7 @@ is_void(CPPType *type) {
 
 /**
  * Returns true if the indicated type is some class that derives from
- * ReferenceCount, or false otherwise.
+ * ReferenceCount, or defines ref and unref(), or false otherwise.
  */
 bool TypeManager::
 is_reference_count(CPPType *type) {
@@ -1459,6 +1459,14 @@ is_reference_count(CPPType *type) {
   case CPPDeclaration::ST_struct:
     {
       CPPStructType *stype = type->as_struct_type();
+
+      // If we have methods named ref() and unref(), this is good enough.
+      if (stype->_scope->_functions.count("ref") &&
+          stype->_scope->_functions.count("unref") &&
+          stype->_scope->_functions.count("get_ref_count")) {
+        return true;
+      }
+
       CPPStructType::Derivation::const_iterator di;
       for (di = stype->_derivation.begin();
            di != stype->_derivation.end();
@@ -2007,6 +2015,46 @@ involves_protected(CPPType *type) {
   }
 }
 
+/**
+ * Returns true if the type involves an rvalue reference.
+ */
+bool TypeManager::
+involves_rvalue_reference(CPPType *type) {
+  switch (type->get_subtype()) {
+  case CPPDeclaration::ST_const:
+    return involves_rvalue_reference(type->as_const_type()->_wrapped_around);
+
+  case CPPDeclaration::ST_reference:
+    return type->as_reference_type()->_value_category == CPPReferenceType::VC_rvalue;
+
+  case CPPDeclaration::ST_pointer:
+    return involves_rvalue_reference(type->as_pointer_type()->_pointing_at);
+
+  case CPPDeclaration::ST_function:
+    {
+      CPPFunctionType *ftype = type->as_function_type();
+      if (involves_rvalue_reference(ftype->_return_type)) {
+        return true;
+      }
+      const CPPParameterList::Parameters &params =
+        ftype->_parameters->_parameters;
+      CPPParameterList::Parameters::const_iterator pi;
+      for (pi = params.begin(); pi != params.end(); ++pi) {
+        if (involves_rvalue_reference((*pi)->_type)) {
+          return true;
+        }
+      }
+      return false;
+    }
+
+  case CPPDeclaration::ST_typedef:
+    return involves_rvalue_reference(type->as_typedef_type()->_type);
+
+  default:
+    return false;
+  }
+}
+
 /**
  * Returns the type this pointer type points to.
  */
