@@ -27,16 +27,18 @@ typedef unsigned long ulong;
 typedef unsigned short ushort;
 typedef unsigned char uchar;
 
-inline namespace std {
 #ifdef _WIN64
-  typedef unsigned long long size_t;
-  typedef long long ssize_t;
-  typedef long long ptrdiff_t;
+#define __SIZE_TYPE__ unsigned long long
+#define __PTRDIFF_TYPE__ long long
 #else
-  typedef unsigned long size_t;
-  typedef long ssize_t;
-  typedef long ptrdiff_t;
+#define __SIZE_TYPE__ unsigned long
+#define __PTRDIFF_TYPE__ long
 #endif
+
+inline namespace std {
+  typedef __SIZE_TYPE__ size_t;
+  typedef __PTRDIFF_TYPE__ ssize_t;
+  typedef __PTRDIFF_TYPE__ ptrdiff_t;
 }
 
 struct timeval;
