@@ -172,9 +172,20 @@ play() {
 
   if (_sd->_sample) {
     push_fresh_buffers();
+    // The macOS implementation of alSourcePlay resets the offset, so we call
+    // it before and avoid calling restart_stalled_audio() afterwards (#1607)
+#ifdef HAVE_OPENAL_FRAMEWORK
+    ALenum status;
+    alGetSourcei(_source, AL_SOURCE_STATE, &status);
+    if (status != AL_PLAYING) {
+      alSourcePlay(_source);
+    }
+#endif
     alSourcef(_source, AL_SEC_OFFSET, _start_time);
     _stream_queued[0]._time_offset = _start_time;
+#ifndef HAVE_OPENAL_FRAMEWORK
     restart_stalled_audio();
+#endif
   } else {
     audio_debug("Play: stream tell = " << _sd->_stream->tell() << " seeking " << _start_time);
     if (_sd->_stream->tell() != _start_time) {
@@ -217,6 +228,8 @@ stop() {
     _stream_queued.resize(0);
   }
 
+  _paused = false;
+
   _manager->stopping_sound(this);
   release_sound_data(false);
 }
@@ -813,11 +826,11 @@ set_active(bool active) {
     } else {
       // ...deactivate the sound.
       if (status()==PLAYING) {
+        stop();
         if (_loop_count==0) {
           // ...we're pausing a looping sound.
           _paused=true;
         }
-        stop();
       }
     }
   }
