@@ -29,6 +29,7 @@
 #include "cmath.h"
 #include "mathNumbers.h"
 #include "geom.h"
+#include "geomLines.h"
 #include "geomTriangles.h"
 #include "geomVertexWriter.h"
 #include "config_mathutil.h"
@@ -229,10 +230,12 @@ test_intersection_from_sphere(const CollisionEntry &entry) const {
   bool intersect;
   LPlane plane;
   LVector3 normal;
+  bool fully_inside = true;
 
   for(ip = 0, intersect = false; ip < 6 && !intersect; ip++) {
     plane = get_plane( ip );
     if (_points[ip].size() < 3) {
+      fully_inside = false;
       continue;
     }
     if (wrt_prev_space != wrt_space) {
@@ -247,6 +250,7 @@ test_intersection_from_sphere(const CollisionEntry &entry) const {
       // moving in the same direction as the plane's normal.
       PN_stdfloat dot = delta.dot(plane.get_normal());
       if (dot > 0.1f) {
+        fully_inside = false;
         continue; // no intersection
       }
 
@@ -303,13 +307,19 @@ test_intersection_from_sphere(const CollisionEntry &entry) const {
     if (!plane.intersects_line(dist, from_center, -(plane.get_normal()))) {
       // No intersection with plane?  This means the plane's effective normal
       // was within the plane itself.  A useless polygon.
+      fully_inside = false;
       continue;
     }
 
-    if (dist > from_radius || dist < -from_radius) {
-      // No intersection with the plane.
+    if (dist > from_radius) {
+      // Fully outside this plane, there can not be an intersection.
+      return nullptr;
+    }
+    if (dist < -from_radius) {
+      // Fully inside this plane.
       continue;
     }
+    fully_inside = false;
 
     LPoint2 p = to_2d(from_center - dist * plane.get_normal(), ip);
     PN_stdfloat edge_dist = 0.0f;
@@ -365,8 +375,9 @@ test_intersection_from_sphere(const CollisionEntry &entry) const {
     }
     intersect = true;
   }
-  if( !intersect )
+  if (!fully_inside && !intersect) {
     return nullptr;
+  }
 
   if (collide_cat.is_debug()) {
     collide_cat.debug()
@@ -947,11 +958,37 @@ fill_viz_geom() {
   tris->add_vertices(3, 7, 0);
   tris->add_vertices(0, 7, 4);
 
-  PT(Geom) geom = new Geom(vdata);
-  geom->add_primitive(tris);
+  PT(GeomLines) lines = new GeomLines(Geom::UH_static);
 
-  _viz_geom->add_geom(geom, get_solid_viz_state());
-  _bounds_viz_geom->add_geom(geom, get_solid_bounds_viz_state());
+  // Bottom
+  lines->add_vertices(0, 1);
+  lines->add_vertices(1, 2);
+  lines->add_vertices(0, 3);
+  lines->add_vertices(2, 3);
+
+  // Top
+  lines->add_vertices(4, 5);
+  lines->add_vertices(5, 6);
+  lines->add_vertices(4, 7);
+  lines->add_vertices(6, 7);
+
+  // Sides
+  lines->add_vertices(0, 4);
+  lines->add_vertices(1, 5);
+  lines->add_vertices(2, 6);
+  lines->add_vertices(3, 7);
+
+  PT(Geom) geom1 = new Geom(vdata);
+  geom1->add_primitive(tris);
+
+  PT(Geom) geom2 = new Geom(vdata);
+  geom2->add_primitive(lines);
+
+  _viz_geom->add_geom(geom1, get_solid_viz_state());
+  _viz_geom->add_geom(geom2, get_wireframe_viz_state());
+
+  _bounds_viz_geom->add_geom(geom1, get_solid_bounds_viz_state());
+  _bounds_viz_geom->add_geom(geom2, get_wireframe_viz_state());
 }
 
 /**
@@ -1412,5 +1449,13 @@ fillin(DatagramIterator& scan, BamReader* manager) {
       v.read_datagram(scan);
       _points[i].push_back(PointDef(p, v));
     }
+    if (size < 4) {
+      LPoint3 array[4];
+      array[0] = _vertex[plane_def[i][0]];
+      array[1] = _vertex[plane_def[i][1]];
+      array[2] = _vertex[plane_def[i][2]];
+      array[3] = _vertex[plane_def[i][3]];
+      setup_points(array, array+4, i);
+    }
   }
 }
