@@ -234,6 +234,8 @@ create_texture(DXScreenData &scrn) {
     case 1:
       if (num_alpha_bits > 0) {
         _d3d_format = D3DFMT_A8;
+      } else if (tex->get_component_type() == Texture::T_float) {
+        _d3d_format = D3DFMT_R32F;
       } else {
         _d3d_format = D3DFMT_L8;
       }
@@ -245,7 +247,11 @@ create_texture(DXScreenData &scrn) {
       _d3d_format = D3DFMT_R8G8B8;
       break;
     case 4:
-      _d3d_format = D3DFMT_A8R8G8B8;
+      if (tex->get_component_type() == Texture::T_float) {
+        _d3d_format = D3DFMT_A32B32G32R32F;
+      } else {
+        _d3d_format = D3DFMT_A8R8G8B8;
+      }
       break;
     }
 
@@ -427,10 +433,26 @@ create_texture(DXScreenData &scrn) {
   }
 
   if (compress_texture) {
-    if (num_color_channels == 1) {
-      CHECK_FOR_FMT(ATI1);
-    } else if (num_alpha_bits == 0 && num_color_channels == 2) {
-      CHECK_FOR_FMT(ATI2);
+    // The ATI1 and ATI2 formats can't be compressed by the driver.
+    // Only choose them if we can compress them on the CPU.
+    // Also, don't choose ATI for a luminance texture, since it gets read as
+    // a texture with just a red channel.
+    if (num_alpha_bits == 0 && !needs_luminance) {
+      if (num_color_channels == 1) {
+        if (scrn._supported_tex_formats_mask & ATI1_FLAG) {
+          if (tex->compress_ram_image(Texture::CM_rgtc)) {
+            target_pixel_format = D3DFMT_ATI1;
+            goto found_matching_format;
+          }
+        }
+      } else if (num_color_channels == 2) {
+        if (scrn._supported_tex_formats_mask & ATI2_FLAG) {
+          if (tex->compress_ram_image(Texture::CM_rgtc)) {
+            target_pixel_format = D3DFMT_ATI2;
+            goto found_matching_format;
+          }
+        }
+      }
     }
     if (num_alpha_bits <= 1) {
       CHECK_FOR_FMT(DXT1);
@@ -486,6 +508,13 @@ create_texture(DXScreenData &scrn) {
       break;
     }
 
+    if (num_color_channels == 1) {
+      CHECK_FOR_FMT(R32F);
+      CHECK_FOR_FMT(X8R8G8B8);
+      CHECK_FOR_FMT(R8G8B8);
+      break;
+    }
+
     if (!((num_color_channels == 3) || (num_color_channels == 4)))
       break; //bail
 
@@ -651,22 +680,7 @@ create_texture(DXScreenData &scrn) {
       }
     }
   case 8:
-    if (needs_luminance) {
-      // don't bother handling those other 8bit lum fmts like 4-4, since 16
-      // 8-8 is usually supported too
-      nassertr(num_color_channels == 1, false);
-
-      // look for native lum fmt first
-      CHECK_FOR_FMT(L8);
-      CHECK_FOR_FMT(L8);
-
-      CHECK_FOR_FMT(R8G8B8);
-      CHECK_FOR_FMT(X8R8G8B8);
-
-      CHECK_FOR_FMT(R5G6B5);
-      CHECK_FOR_FMT(X1R5G5B5);
-
-    } else if (num_alpha_bits == 8) {
+    if (num_alpha_bits == 8) {
       // look for 16bpp A8L8, else 32-bit ARGB, else 16-4444.
 
       // skip 8bit alpha only (D3DFMT_A8), because I think only voodoo
@@ -677,6 +691,21 @@ create_texture(DXScreenData &scrn) {
       CHECK_FOR_FMT(A8L8);
       CHECK_FOR_FMT(A8R8G8B8);
       CHECK_FOR_FMT(A4R4G4B4);
+    } else {
+      if (needs_luminance) {
+        // don't bother handling those other 8bit lum fmts like 4-4, since 16
+        // 8-8 is usually supported too
+        nassertr(num_color_channels == 1, false);
+
+        // look for native lum fmt first
+        CHECK_FOR_FMT(L8);
+      }
+
+      CHECK_FOR_FMT(R8G8B8);
+      CHECK_FOR_FMT(X8R8G8B8);
+
+      CHECK_FOR_FMT(R5G6B5);
+      CHECK_FOR_FMT(X1R5G5B5);
     }
     break;
 
@@ -1533,8 +1562,8 @@ d3d_surface_to_texture(RECT &source_rect, IDirect3DSurface9 *d3d_surface,
 
       for (DWORD y = 0; y < copy_height; y++) {
         source_word = ((DWORD*)surface_bytes) + x_window_offset;
-        memcpy(dest_line, source_word, byte_pitch);
-        dest_line += byte_pitch;
+        memcpy(dest_line, source_word, copy_width * 4);
+        dest_line += copy_width * 4;
         surface_bytes += byte_pitch;
       }
     } else {
@@ -1584,8 +1613,8 @@ d3d_surface_to_texture(RECT &source_rect, IDirect3DSurface9 *d3d_surface,
       // 24bpp texture case (numComponents == 3)
       for (DWORD y = 0; y < copy_height; y++) {
         source_byte = surface_bytes + x_window_offset * 3 * sizeof(BYTE);
-        memcpy(dest_byte, source_byte, byte_pitch);
-        dest_byte += byte_pitch;
+        memcpy(dest_byte, source_byte, copy_width * 3);
+        dest_byte += copy_width * 3;
         surface_bytes += byte_pitch;
       }
     }
@@ -1815,7 +1844,28 @@ HRESULT DXTextureContext9::fill_d3d_texture_mipmap_pixels(int mip_level, int dep
     source_row_byte_length = width * sizeof(USHORT);
     pixels = (BYTE*)temp_buffer;
   }
-  else if (component_width != 1) {
+  else if (_d3d_format == D3DFMT_A32B32G32R32F && source_format == D3DFMT_A32B32G32R32F) {
+    size_t total_components = (size_t)width * (size_t)height * 4;
+    float *temp_buffer = new float[total_components];
+    if (!IS_VALID_PTR(temp_buffer)) {
+      dxgsg9_cat.error()
+        << "FillDDTextureMipmapPixels couldnt alloc mem for temp pixbuf!\n";
+      goto exit_FillMipmapSurf;
+    }
+    using_temp_buffer = true;
+
+    // Swap red and blue components.
+    float *out_pixels = (float *)temp_buffer;
+    const float *source_pixels = (const float *)pixels;
+    for (size_t i = 0; i < total_components; i += 4) {
+      out_pixels[i] = source_pixels[i + 2];
+      out_pixels[i + 1] = source_pixels[i + 1];
+      out_pixels[i + 2] = source_pixels[i + 0];
+      out_pixels[i + 3] = source_pixels[i + 3];
+    }
+    pixels = (BYTE*)temp_buffer;
+  }
+  else if (component_width != 1 && _d3d_format != D3DFMT_R32F) {
     // Convert from 16-bit per channel (or larger) format down to 8-bit per
     // channel.  This throws away precision in the original image, but dx8
     // doesn't support high-precision images anyway.
@@ -2187,8 +2237,29 @@ fill_d3d_volume_texture_pixels(DXScreenData &scrn) {
     source_row_byte_length = orig_width * sizeof(USHORT);
     source_page_byte_length = orig_height * source_row_byte_length;
     pixels = (BYTE*)temp_buffer;
+  }
+  else if (_d3d_format == D3DFMT_A32B32G32R32F && source_format == D3DFMT_A32B32G32R32F) {
+    size_t total_components = (size_t)orig_width * (size_t)orig_height * (size_t)orig_depth * 4;
+    float *temp_buffer = new float[total_components];
+    if (!IS_VALID_PTR(temp_buffer)) {
+      dxgsg9_cat.error()
+        << "FillDDSurfaceTexturePixels couldnt alloc mem for temp pixbuf!\n";
+      goto exit_FillDDSurf;
+    }
+    using_temp_buffer = true;
 
-  } else if (component_width != 1) {
+    // Swap red and blue components.
+    float *out_pixels = (float *)temp_buffer;
+    const float *source_pixels = (const float *)pixels;
+    for (size_t i = 0; i < total_components; i += 4) {
+      out_pixels[i] = source_pixels[i + 2];
+      out_pixels[i + 1] = source_pixels[i + 1];
+      out_pixels[i + 2] = source_pixels[i + 0];
+      out_pixels[i + 3] = source_pixels[i + 3];
+    }
+    pixels = (BYTE*)temp_buffer;
+  }
+  else if (component_width != 1 && _d3d_format != D3DFMT_R32F) {
     // Convert from 16-bit per channel (or larger) format down to 8-bit per
     // channel.  This throws away precision in the original image, but dx8
     // doesn't support high-precision images anyway.
@@ -2334,6 +2405,9 @@ get_bits_per_pixel(Texture::Format format, int *alphbits) {
     *alphbits = 32;
     return 128;
 
+  case Texture::F_r32:
+    return 32;
+
   case Texture::F_srgb:
     return 24;
   case Texture::F_srgb_alpha:
