@@ -259,7 +259,7 @@ cleanup_polygon_indices(vector_int &polygon) {
       ++pi;
     } else {
       // This index is out-of-bounds; remove it.
-      polygon.erase(_polygon.begin() + pi);
+      polygon.erase(polygon.begin() + pi);
     }
   }
 
@@ -271,14 +271,46 @@ cleanup_polygon_indices(vector_int &polygon) {
       ++pi;
     } else {
       // This vertex repeats the previous one; remove it.
-      polygon.erase(_polygon.begin() + pi);
+      polygon.erase(polygon.begin() + pi);
     }
   }
 
-  if (polygon.size() > 1 && _vertices[polygon.back()] == _vertices[_polygon.front()]) {
+  if (polygon.size() > 1 && _vertices[polygon.back()] == _vertices[polygon.front()]) {
     // The last vertex repeats the first one; remove it.
     polygon.pop_back();
   }
+
+  // Another pass over the polygons, this time removing any "tails".
+  while (polygon.size() >= 3) {
+    bool removed_any = false;
+
+    int prevprev = polygon[polygon.size() - 2];
+    int prev = polygon[polygon.size() - 1];
+
+    for (size_t i = 0; i < polygon.size(); ++i) {
+      int cur = polygon[i];
+      if (_vertices[prevprev] == _vertices[cur]) {
+        // Cut off the tail.
+        removed_any = true;
+        polygon.erase(polygon.begin() + i);
+        if (i == 0) {
+          polygon.pop_back();
+        } else {
+          polygon.erase(polygon.begin() + i - 1);
+        }
+        break;
+      }
+
+      prevprev = prev;
+      prev = cur;
+    }
+
+    // This might have been the tip of a longer tail, so if we removed
+    // something, go again.
+    if (!removed_any) {
+      break;
+    }
+  }
 }
 
 
@@ -2185,6 +2217,8 @@ triangulate_single_polygon(int nvert, int posmax, int side) {
       endv = mchain[posmax].vnum;
     }
 
+  int num_triangles = 0;
+
   while ((v != endv) || (ri > 1))
     {
       // cerr << " v = " << v << " ri = " << ri << " rc = " << rc.size() << "
@@ -2199,8 +2233,13 @@ triangulate_single_polygon(int nvert, int posmax, int side) {
                                     vert[rc[ri]].pt);
           if ( crossResult >= 0 )  /* could be convex corner or straight */
             {
-              if ( crossResult > 0)  /* convex corner: cut it off */
+              if (crossResult > 0) {  /* convex corner: cut it off */
                 _result.push_back(Triangle(this, rc[ri - 1], rc[ri], v));
+                if (++num_triangles >= nvert - 2) {
+                  // We can't generate more than this number of triangles.
+                  return;
+                }
+              }
               /* else : perfectly straight, will be abandoned anyway */
               ri--;
               rc.pop_back();
