@@ -41,6 +41,8 @@
 #include "pta_LVecBase2.h"
 #include "stl_compares.h"
 #include "shaderInput.h"
+#include "internalNameCollection.h"
+#include "materialCollection.h"
 #include "textureCollection.h"
 #include "textureStageCollection.h"
 
@@ -49,13 +51,9 @@ class FindApproxPath;
 class FindApproxLevelEntry;
 class Light;
 class PolylightNode;
-class InternalNameCollection;
 class Texture;
 class TextureStage;
-class TextureCollection;
-class TextureStageCollection;
 class Material;
-class MaterialCollection;
 class Fog;
 class GlobPattern;
 class PreparedGraphicsObjects;
@@ -621,6 +619,9 @@ PUBLISHED:
   Texture *get_texture() const;
   Texture *get_texture(TextureStage *stage) const;
   void replace_texture(Texture *tex, Texture *new_tex);
+#ifdef CPPPARSER  // Let interrogate know this also accepts None
+  void replace_texture(Texture *tex, std::nullptr_t new_tex);
+#endif
   const SamplerState &get_texture_sampler() const;
   const SamplerState &get_texture_sampler(TextureStage *stage) const;
 
@@ -773,6 +774,9 @@ PUBLISHED:
   bool has_material() const;
   PT(Material) get_material() const;
   void replace_material(Material *mat, Material *new_mat);
+#ifdef CPPPARSER  // Let interrogate know this also accepts None
+  void replace_material(Material *mat, std::nullptr_t new_mat);
+#endif
 
   void set_fog(Fog *fog, int priority = 0);
   void set_fog_off(int priority = 0);
@@ -954,6 +958,10 @@ PUBLISHED:
   static NodePath decode_from_bam_stream(vector_uchar data, BamReader *reader = nullptr);
 
 private:
+  bool replace_copied_nodes(const NodePath &source, const NodePath &dest,
+                            const PandaNode::InstanceMap &inst_map,
+                            Thread *current_thread);
+
   static NodePathComponent *
   find_common_ancestor(const NodePath &a, const NodePath &b,
                        int &a_count, int &b_count,
@@ -1054,6 +1062,8 @@ private:
 
 INLINE std::ostream &operator << (std::ostream &out, const NodePath &node_path);
 
+#include "nodePathCollection.h"
+
 #include "nodePath.I"
 
 #endif
