@@ -107,7 +107,7 @@ init() {
   // can't safely load the file.
   if (_file_major != _bam_major_ver ||
       _file_minor < _bam_first_minor_ver ||
-      _file_minor > _bam_minor_ver) {
+      _file_minor > _bam_last_minor_ver) {
     bam_cat.error()
       << "Bam file is version " << _file_major << "." << _file_minor
       << ".\n";
@@ -120,7 +120,7 @@ init() {
       bam_cat.error()
         << "This program can only load version "
         << _bam_major_ver << "." << _bam_first_minor_ver << " through "
-        << _bam_major_ver << "." << _bam_minor_ver << " bams.\n";
+        << _bam_major_ver << "." << _bam_last_minor_ver << " bams.\n";
     }
 
     return false;
@@ -315,7 +315,7 @@ read_object(TypedWritable *&ptr, ReferenceCount *&ref_ptr) {
  * time to call it.
  *
  * This must be called at least once after reading a particular object via
- * get_object() in order to validate that object.
+ * read_object() in order to validate that object.
  *
  * The return value is true if all objects have been resolved, or false if
  * some objects are still outstanding (in which case you will need to call
