@@ -15,6 +15,7 @@
 #define CONFIG_ASSIMP_H
 
 #include "pandatoolbase.h"
+#include "notifyCategoryProxy.h"
 #include "configVariableBool.h"
 #include "configVariableDouble.h"
 #include "dconfig.h"
@@ -32,6 +33,7 @@ extern ConfigVariableBool assimp_optimize_graph;
 extern ConfigVariableBool assimp_flip_winding_order;
 extern ConfigVariableBool assimp_gen_normals;
 extern ConfigVariableDouble assimp_smooth_normal_angle;
+extern ConfigVariableBool assimp_collapse_dummy_root_node;
 
 extern EXPCL_ASSIMP void init_libassimp();
 
