@@ -38,6 +38,15 @@ void PandaLogger::OnDebug(const char *message) {
   assimp_cat.debug() << message << "\n";
 }
 
+/**
+ *
+ */
+void PandaLogger::OnVerboseDebug(const char *message) {
+  if (assimp_cat.is_spam()) {
+    assimp_cat.spam() << message << "\n";
+  }
+}
+
 /**
  *
  */
