@@ -15,6 +15,7 @@
 #include "gtkStatsMonitor.h"
 #include "pStatCollectorDef.h"
 #include "numeric_types.h"
+#include "string_utils.h"
 
 static const int default_strip_chart_width = 400;
 static const int default_strip_chart_height = 100;
@@ -120,7 +121,7 @@ new_data(int thread_index, int frame_number) {
   if (!_pause) {
     update();
 
-    std::string text = format_number(get_average_net_value(), get_guide_bar_units(), get_guide_bar_unit_name());
+    std::string text = get_total_text();
     if (_net_value_text != text) {
       _net_value_text = text;
       gtk_label_set_text(GTK_LABEL(_total_label), _net_value_text.c_str());
@@ -402,20 +403,23 @@ set_drag_mode(GtkStatsGraph::DragMode drag_mode) {
 gboolean GtkStatsStripChart::
 handle_button_press(GtkWidget *widget, int graph_x, int graph_y,
         bool double_click) {
-  if (double_click) {
-    // Double-clicking on a color bar in the graph is the same as double-
-    // clicking on the corresponding label.
-    clicked_label(get_collector_under_pixel(graph_x, graph_y));
-    return TRUE;
+  if (graph_x >= 0 && graph_y >= 0 && graph_x < get_xsize() && graph_y < get_ysize()) {
+    if (double_click) {
+      // Double-clicking on a color bar in the graph is the same as double-
+      // clicking on the corresponding label.
+      clicked_label(get_collector_under_pixel(graph_x, graph_y));
+      return TRUE;
+    }
+
+    if (_potential_drag_mode == DM_none) {
+      set_drag_mode(DM_scale);
+      _drag_scale_start = pixel_to_height(graph_y);
+      // SetCapture(_graph_window);
+      return TRUE;
+    }
   }
 
-  if (_potential_drag_mode == DM_none) {
-    set_drag_mode(DM_scale);
-    _drag_scale_start = pixel_to_height(graph_y);
-    // SetCapture(_graph_window);
-    return TRUE;
-
-  } else if (_potential_drag_mode == DM_guide_bar && _drag_guide_bar >= 0) {
+  if (_potential_drag_mode == DM_guide_bar && _drag_guide_bar >= 0) {
     set_drag_mode(DM_guide_bar);
     _drag_start_y = graph_y;
     // SetCapture(_graph_window);
@@ -455,7 +459,8 @@ handle_button_release(GtkWidget *widget, int graph_x, int graph_y) {
  */
 gboolean GtkStatsStripChart::
 handle_motion(GtkWidget *widget, int graph_x, int graph_y) {
-  if (_drag_mode == DM_none && _potential_drag_mode == DM_none) {
+  if (_drag_mode == DM_none && _potential_drag_mode == DM_none &&
+      graph_x >= 0 && graph_y >= 0 && graph_x < get_xsize() && graph_y < get_ysize()) {
     // When the mouse is over a color bar, highlight it.
     _label_stack.highlight_label(get_collector_under_pixel(graph_x, graph_y));
 
